import com.sun.java.util.collections.*; //change for 1.2!
//import java.util.*;
import java.io.*;
import uk.ac.leeds.ccg.geotools.*;

public class Database extends PointLayer
{
    // Members
    Vector points = new Vector();
    Vector cases = new Vector();
    SimpleGeoData geoc = new SimpleGeoData();
    double overrat;
    double minX=Double.POSITIVE_INFINITY;
    double minY=Double.POSITIVE_INFINITY;
    double maxX=Double.NEGATIVE_INFINITY;
    double maxY=Double.NEGATIVE_INFINITY;
    GeoRectangle b = new GeoRectangle();
		String name = "unamed datafile";

    // Constructors
    public Database()
    {
	useQuadtree=true;
	geoc.setMissingValueCode(0.0);
    }
    public Database(String name)
    {
			this(new File(name));
		}
		public Database(File name){
			useQuadtree=true;
			geoc.setMissingValueCode(0.0);
			loadText(name);
    }

    // Methods
    public Vector getCases() {return cases;}
    public GeoData getCaseVals() {return geoc;}
    public void setup(Vector pv,double overrate)
    {
	b=new GeoRectangle();
	MyPoint p; 
	int ncases=0;
	int npop=0;
	for(int i=0;i<pv.size();i++)
	    {
		p=(MyPoint)pv.elementAt(i);
		maxX=Math.max(p.getX(),maxX);
		minX=Math.min(p.getX(),minX);
		maxY=Math.max(p.getY(),maxY);
		minY=Math.min(p.getY(),minY);
		npop+=p.pop;
		p.setPop(p.getPop()* overrate);
		b.add(p);
		if(p.cases>0)
		    { 
			geoc.setValue(p.getID(),(double)p.cases);
			cases.addElement(p);
			ncases+=p.cases;
		    }
	    }
	overrat = (double)ncases/(double)npop;
	System.out.println("setup "+pv.size()+" pop= "+npop+" cases "+ncases+
			" Rate = "+overrat+" "+overrate);
	shapeTree.setBounds(b);
	for(int i=0;i<pv.size();i++)
	    {
		if(i%1000==0)
		    {
			System.out.println("added "+i);
		    }
		addGeoPoint((MyPoint)pv.elementAt(i));
	    }
	points=pv;
    }	
    public void loadText(String name) {
			loadText(new File(name));
		}
		public void loadText(File name){
		this.name = name.toString();
	StreamTokenizer xyStrTok;
	BufferedReader xyInput;
	try
	    {
		xyInput = new BufferedReader(new FileReader(name));
		xyStrTok = new  StreamTokenizer(xyInput);
		xyStrTok.parseNumbers();
		xyStrTok.whitespaceChars(',',',');
		xyStrTok.eolIsSignificant(false);
		int token;
		int id;
		double x,y;
		int pop,tcase;
		int npop=0;
		int ncases=0;
		MyPoint p;
		token=xyStrTok.nextToken();
		while(token!=xyStrTok.TT_EOF)
		    {
			id= (int)xyStrTok.nval;
			token=xyStrTok.nextToken();
			x = xyStrTok.nval;
			maxX=Math.max(x,maxX);
			minX=Math.min(x,minX);
			token=xyStrTok.nextToken();
			y = xyStrTok.nval;
			maxY=Math.max(y,maxY);
			minY=Math.min(y,minY);
			token=xyStrTok.nextToken();
			tcase = (int)xyStrTok.nval;
			ncases+=tcase;
			token=xyStrTok.nextToken();
			pop = (int)xyStrTok.nval;
			npop+=pop;
			token=xyStrTok.nextToken();
			p = new MyPoint(id,x,y,tcase,pop);
			b.add(p);
			points.addElement(p);
			if(tcase>0)
			    { 
				geoc.setValue(id,(double)tcase);
				cases.addElement(p);
			    }
		    }
		shapeTree.setBounds(b);
		for(int i=0;i<points.size();i++)
		    {
			addGeoPoint((MyPoint)points.elementAt(i));
		    }
		overrat = (double)ncases/(double)npop;
		System.out.println("Read "+points.size()+" pop= "+npop+
				   " cases "+ncases+" Rate = "+overrat);
	    }
	catch(Exception e) {System.err.println("Database load error:\n"+e);}
    }
    public void convertToRates()
    {
	MyPoint p;
		for(int i=0;i<points.size();i++)
		    {
			p=(MyPoint)points.elementAt(i);
			p.setPop(p.getPop()* overrat);
		    }
    }
    public final int totalObs() {return points.size();}
    public final int getTotalObs() {return points.size();}
    public final Vector getPoints(GeoRectangle gb)
    {
	int [] m = getIDs(gb,0);
	Vector results = new Vector(m.length);
	for(int i=0;i<m.length;i++)
	    {
		results.addElement(shapeList.elementAt(i));
	    }
	return results;
    }
    public final Vector getPoints(GeoCircle gc)
    {
	//System.out.println("checking "+gc);
	//Vector results = getShapesIn(gc.getBounds(),0);
	HashSet results = new HashSet(shapeTree.find(gc.getBounds()));
	if(results.size()==0) return new Vector(0);
	Vector res = new Vector(results.size());
	//if(results.size()>0)System.out.println("get Points - got "+results.size()+" "+gc.getRadius());
	GeoPoint p;
	Iterator it = results.iterator();
	while(it.hasNext())
	    {
		p=(GeoPoint)it.next();
		if(gc.contains(p))
		    {
			res.addElement(p);
		    }
	    }
	//System.out.println("Size "+res.size());
	return res;
    }
    public Vector getShapesIn(GeoRectangle box,int mode)
    {
	Vector list = new Vector();
	//System.out.println(""+box);
	for(int i = 0;i < shapeList.size();i++)
	    {
		MyPoint temp = (MyPoint)shapeList.elementAt(i);
		//System.out.println("Checking "+temp.x+" "+temp.y);
		if(box.contains(temp))
		    {
			//System.out.println("hit "+temp);
			list.addElement(temp);
		    }
	    }
	return list;
	/*int[] IDs = new int[list.size()];
	  if(list.size()==0) return IDs;
	  //System.out.println("size "+list.size());
	  for(int i=0;i<list.size();i++){
	  GeoShape temp = (GeoShape)list.elementAt(i);
	  //System.out.println(" "+temp.getID());
	  IDs[i]=temp.getID();
	  }
		return IDs;*/
    }
    public void save(File f)
    {
	MyPoint temp;
	try
	    {	
		DataOutputStream dos = new DataOutputStream(new
		    FileOutputStream(f));
		System.out.println("saving "+points.size()+" to "+f);
		for(int i = 0;i < points.size();i++)
		    {
			temp = (MyPoint)points.elementAt(i);
			dos.writeInt(temp.getID());
			dos.writeDouble(temp.x);
			dos.writeDouble(temp.y);
			dos.writeInt(temp.cases);
			dos.writeDouble(temp.pop);
		    }
		dos.close();
	    }
	catch(IOException ie){
	    System.err.println("Error in save\n"+ie);
	    return;
	}
    }
    public void load(File f)
    {
		this.name = f.toString();
	MyPoint temp;
	b=new GeoRectangle();
	int id,c;
	double x,y,p;
	try
	    {
		DataInputStream dis = new DataInputStream
		    (new FileInputStream(f));
		while(dis.available()>0)
		    {
		    id=dis.readInt();
		    x=dis.readDouble();
		    y=dis.readDouble();
		    c=dis.readInt();
		    p=dis.readDouble();
		    temp = new MyPoint(id,x,y,c,p);
		    points.addElement(temp);
		    b.add(temp);
		    if(c>0)
			{
			    cases.addElement(temp);
			    geoc.setValue(id,(double)c);
			}
		    }
	    }
	catch(IOException ie)
	    {
		System.err.println("Error on database load\n"+ie);
		return;
	    }
	shapeTree.setBounds(b);
	for(int i=0;i<points.size();i++)
	    {
		addGeoPoint((MyPoint)points.elementAt(i));
	    }
    }    
		public int getSize(){
			return points.size();
		}
		public int getNumberCases(){
			return cases.size();
		}
		public void setName(String n){
			name=n;
		}
		public String getName(){
			return name;
		}
}
