/*
 * ThemeStack.java
 *
 * Created on 04 October 2001, 01:19
 */

package uk.ac.leeds.ccg.geotools;
import java.util.*;
import java.util.Iterator;

/**
 *
 * @author  jamesm
 * @version
 */
public class ThemeStack {
    HashMap lookup = new HashMap();
    TreeSet themeInfos = new TreeSet();
    Viewer view;
    /** Creates new ThemeStack */
    public ThemeStack(Viewer v) {
        view=v;
    }
    
    public int addTheme(Theme t,int weight,boolean visible){
        while(getThemeByWeight(weight)!=null){
            weight++;
        }
        //System.out.println("Added new theme to stack with weight of "+weight);
        ThemeInfo info = new ThemeInfo(t,weight,visible);
        lookup.put(t,info);
        themeInfos.add(info);
        view.invalidateSequenceBuffer();
        view.repaint();
        return weight;
    }
    
    public void removeTheme(Theme t){
        themeInfos.remove(lookup.get(t));
        lookup.remove(t);
        view.invalidateSequenceBuffer();
        view.repaint();
    }
    
    public void setToBottom(Theme t){
        //ThemeInfo info = (ThemeInfo)lookup.get(t);
        //themeInfos.remove(info);
        //info.setWeight(((ThemeInfo)themeInfos.last()).getWeight()+1);
        setWeight(t,((ThemeInfo)themeInfos.first()).getWeight()+1);
        view.invalidateSequenceBuffer();
        view.repaint();
    }
    
    public void setToTop(Theme t){
        ThemeInfo info = (ThemeInfo)lookup.get(t);
        themeInfos.remove(info);
        ((ThemeInfo)themeInfos.first()).getWeight();
        info.setWeight(((ThemeInfo)themeInfos.last()).getWeight()-1);
        themeInfos.add(info);
        view.invalidateSequenceBuffer();
        view.repaint();
    }
    
    
    public void setWeight(Theme t,int weight){
        Theme old = this.getThemeByWeight(weight);
        if(old!=null)setWeight(old,weight+1);
        //System.out.println("Setting weight of "+((ThemeInfo)lookup.get(t)).getTheme()+" to "+weight);
        ThemeInfo info = (ThemeInfo)lookup.get(t);
        themeInfos.remove(info);
        info.setWeight(weight);
        themeInfos.add(info);
        view.invalidateSequenceBuffer();
        view.repaint();
    }
    
    public void swapThemes(Theme a,Theme b){
        //System.out.println("Swapping theme "+a.getName()+" with "+b.getName());
        ThemeInfo first = (ThemeInfo)lookup.get(a);
        ThemeInfo second = (ThemeInfo)lookup.get(b);
        int aw = first.getWeight();
        int bw = second.getWeight();
        
        themeInfos.remove(first);
        themeInfos.remove(second);
        second.setWeight(aw);
        first.setWeight(bw);
        themeInfos.add(first);
        themeInfos.add(second);
        view.invalidateSequenceBuffer();
        view.repaint();
    }
    
    public void swapThemes(int a,int b){
        swapThemes(getThemeByWeight(a),getThemeByWeight(b));
    }
    
    public int getWeight(Theme t){
        return ((ThemeInfo)lookup.get(t)).getWeight();
    }
    
    public void setIsVisible(Theme t,boolean visible){
        ((ThemeInfo)lookup.get(t)).setIsVisible(visible);
        view.invalidateSequenceBuffer();
        view.repaint();
    }
    
    public boolean isVisible(Theme t){
        return ((ThemeInfo)lookup.get(t)).isVisible();
    }
    
    protected ThemeInfo[] getOrderedThemeInfos(){
        //SortedMap head = themeInfos.headSet(new ThemeInfo(null,Integer.MAX_VALUE,false));
        return (ThemeInfo[])themeInfos.toArray(new ThemeInfo[0]);
    }
    
    public Theme getThemeByWeight(int w){
        ThemeInfo dummy = new ThemeInfo(null,w,true);
        ThemeInfo dummy2 = new ThemeInfo(null,w-1,true);
        SortedSet sub = themeInfos.subSet(dummy,dummy2);
        if(sub.isEmpty())return null;
        return ((ThemeInfo)sub.first()).getTheme();
        
    }
    
    /** return the bounding box of the themes in the stack */
    public GeoRectangle getBounds(){
        GeoRectangle r = new GeoRectangle();
        Set themes = lookup.keySet();
        Iterator it = themes.iterator();
        while(it.hasNext()){
            r.add(((Theme)it.next()).getBounds());
        }
        return r;
    }
    
    
    class ThemeInfo implements Comparable{
        Theme theme;
        boolean visible;
        int weight;
        
        public ThemeInfo(Theme theme,int weight,boolean visible){
            this.theme=theme;
            this.weight=weight;
            this.visible = visible;
        }
        
        public int compareTo(Object o){
            if(o == null) throw new IllegalArgumentException("Null object in ThemeInfo.compare");
            ThemeInfo b = (ThemeInfo)o;
            //if(!visible && b.visible) return -1000;
            //else if(visible &&!b.visible) return 1000;
            //else return weight-b.weight;
            return b.weight-weight;
        }
        
        public boolean isVisible(){
            return visible;
        }
        public int getWeight(){
            return weight;
        }
        
        public Theme getTheme(){
            return theme;
        }
        
        public void setWeight(int weight){
            this.weight = weight;
        }
        
        public void setIsVisible(boolean visible){
            this.visible = visible;
        }
    }
}
