/*
 * GeoTools java GIS tookit (c) The Centre for Computational Geography 2002
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation version 2.1
 */

package uk.ac.leeds.ccg.widgets;

import uk.ac.leeds.ccg.geotools.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;

/**
 * A button used to pan the viewer by fixed increment 
 *
 * @author Sylvain Pasche <sylvain_pasche@yahoo.fr>
 */
public class PanButton extends Button implements ActionListener {

    public final static int DIR_NW = 0;
    public final static int DIR_N  = 1;
    public final static int DIR_NE = 2;
    public final static int DIR_W  = 3;
    public final static int DIR_E  = 4;
    public final static int DIR_SW = 5;
    public final static int DIR_S  = 6;
    public final static int DIR_SE = 7;

    int direction;
    double panPercent = .2;
    Viewer viewer;
    /**
     * Construct a panning button with a label name
     *
     * @param label the buttons's laber
     * @param viewer the viewer displaying the map
     * @param direction a constant for which direction to pan
     */
    public PanButton(String label, Viewer viewer, int direction) {
        if (label == null) {

            switch(direction) {
            case DIR_NW: label = "NW"; break;
            case DIR_N : label = "N" ; break;
            case DIR_NE: label = "NE"; break;
            case DIR_W : label = "W" ; break;
            case DIR_E : label = "E" ; break;
            case DIR_SW: label = "SW"; break;
            case DIR_S : label = "S" ; break;
            case DIR_SE: label = "SE"; break;
            }
        }

        setLabel(label);
        this.direction = direction;
        this.viewer = viewer;
        addActionListener(this);
    }

    /**
     * Construct a panning button with a default label name
     *
     * @param viewer the viewer displaying the map
     * @param direction a constant for which direction to pan
     */
    public PanButton(Viewer viewer, int direction) {

        this(null, viewer, direction);
    }
    /**
     * Sets button's pan percent
     *
     * @param percent percents of the map which will still be visible after pan
     */
    public void setPanPercent(double percent) {
        panPercent = percent;
    }
    /**
     * Gets button's pan percent
     *
     */
    public double getPanPercent() {
        return panPercent;
    }

    public void actionPerformed(ActionEvent evt) {

        System.out.println("Panning " + evt.getActionCommand());
        double horizMove = 0.0;
        double vertMove = 0.0;
        switch (direction) {
        case DIR_NW:
            horizMove = -1.0;
            vertMove = 1.0;
            break;
        case DIR_N:
            vertMove = 1.0;
            break;
        case DIR_NE:
            horizMove = 1.0;
            vertMove = 1.0;
            break;
        case DIR_W:
            horizMove = -1.0;
            break;
        case DIR_E:
            horizMove = 1.0;
            break;
        case DIR_SW:
            horizMove = -1.0;
            vertMove = -1.0;
            break;
        case DIR_S:
            vertMove = -1.0;
            break;
        case DIR_SE:
            horizMove = 1.0;
            vertMove = -1.0;
            break;
        }
        horizMove *= panPercent;
        vertMove *= panPercent;

        GeoRectangle origExtent = viewer.getScale().getProjectedMapExtent();
        GeoRectangle newExtent = 
            new GeoRectangle(origExtent.x + horizMove * origExtent.width,
                             origExtent.y + vertMove * origExtent.height,
                             origExtent.width,
                             origExtent.height);
        viewer.getScale().setProjectedMapExtent(newExtent);
    }
}
