package uk.ac.leeds.ccg.cluster;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
//import com.sun.java.util.collections.*; //change for 1.2!
import uk.ac.leeds.ccg.geotools.*;

/** A class that implements Besag and Newell Cluster Hunt
 * The main class BandN creates the main frame and proivides 3 buttons
 * modified to use the new results object - 31/01/01
 */
public class Knn extends SimpleMethod implements Method, Runnable
{
    // Members
		KnnParameters parameters;
    KnnParametersViewer parametersViewer;
    Database database;
    Results results;
    Viewer viewer;                        // Geotools viewer
    GeoRectangle geoRectangle = null;     // Geotools geoRectangle
    Shader shader;                        // Geotools shader
    
    // Methods
    public static void main(String args[]){
        Knn knn = new Knn();
        knn.run();
    }
    
		public Knn(){
			parameters = new KnnParameters();
			parametersViewer = new KnnParametersViewer(parameters);

		}
    public String toString() {return "Knn";}
    public Parameters getParameters() {return parameters;}
    public ParametersViewer getParametersViewer() {return parametersViewer;}
    public void setShader(Shader s) {shader=s;}
    public void setViewer(Viewer v) {viewer=v;}
    public void setData(Database d) {database=d;}

    public SignificanceTest getSignificanceTest(){return significanceTest;}
    
    public void setSearch(GeoRectangle r) {geoRectangle=r;}
    public Results getResults(){return results;}
    
    public void run(){
				results = new Results();
				int k = parameters.getK();
        MyPoint p;
        // MyDouble statis = new MyDouble();
        double stat = (double) 0.0;
				System.out.println("Knn "+significanceTest);
        if (significanceTest == null) {
						System.out.println("Knn started with a null sig test using Poisson");
            significanceTest = new PoissonTest();
            significanceTest.setParameters(parameters);
        }
        System.out.println(""+this+" "+significanceTest);
        if(geoRectangle==null) geoRectangle=database.getBounds();
        Vector tcases = database.getCases();
        Vector cases= new Vector();
        for(int i=0;i<tcases.size();i++){
            p=(MyPoint)tcases.elementAt(i);
            if(geoRectangle.contains(p)) cases.addElement(p);
        }
        int id=0;
        // Go through the shapes vector that the canvas holds and extract the points
        // Calculate the overall rate of the disease
        // Set array dist to store the distance from a case point to all others case points
        double[] dist = new double[cases.size()];
        for(int i=0;i<cases.size();i++){
            notifyProgressEventListener((double)i*100.0/(double)cases.size());
            // Set point p to be the element of the case vector at i
            p=(MyPoint)cases.elementAt(i);
            // Get the distance from p to all other cases
            for(int j=0;j<cases.size();j++){
                // If points are different get the distance to the jth point
                if(i!=j){
                    dist[j]=p.dist((MyPoint)cases.elementAt(j));
                }
                else{
                    // If not set the distance to infinity
                    dist[j]=Double.POSITIVE_INFINITY;
                }
            }
            // Set top array to store the k smallest distances
            double top[] = new double[k];
            for(int kk=0;kk<k;kk++){
                // Set it to something high (this will fail if K < number of points)
                top[kk]=Double.POSITIVE_INFINITY;
                int index=-1;
                // go through the whole array of distances
                for(int j=0;j<cases.size();j++){
                    // If the current distance is less than the best so far
                    // copy it and keep a note of where it was (index)
                    if(top[kk]>dist[j]){
                        top[kk]=dist[j];
                        index=j;
                    }
                }
                // Replace the shortest distance with infinity so that next
                // we find the next shortest.
                dist[index]=Double.POSITIVE_INFINITY;
            }
            // Create a circle with a radius of the distance to the kth point
            GeoCircle cc = new GeoCircle(id++,p.x,p.y,(int)top[k-1]);
            // Create a Poisson test object
            // MyPoint p2;
            // int totcase=0;
            // double totpop=0;
            // Go through the vector of cases and add up all the cases with in
            // the circle using the contains method
            // for(int j=0;j<cases.size();j++){
            // p2=(MyPoint)cases.elementAt(j);
            // if(cc.contains(p2))
            // totcase+=p2.getCases();
            // }
            // Go through the cases vector and add up the pop with in the circle
            Vector points = database.getPoints(cc);
            // for(int j=0;j<points.size();j++){
            // p2=(MyPoint)points.elementAt(j);
            // totpop+=p2.getPop();
            // }
            // If significant pass details to the results object
            // if(significanceTest.isSignificant((totpop),totcase,statis))
						try{
							if(significanceTest.isSignificant(points))
							{
									stat = significanceTest.getStatistic();
									// results.addResult(cc,statis.x);
									results.addResult(cc,stat);
							}
						}catch(SignificanceException e){
							System.out.println(""+e);
						}
            // stat = significanceTest.getStatis();
            // System.out.println(i +"  "+  stat);
        }
        notifyProgressEventListener(0.0);
    }
}
